==> squidwall-0.3b.tar.bz2: squidwall-0.3b/squidwall.c <==
#include "squidwall.h"

void setup(void);

/* fd of logfile */
int64 fd_log;

/* ip's and ports */
struct ip_address clamd_ip;
struct ip_address proxy_ip;
unsigned clamd_port;
unsigned clams_port;
unsigned proxy_port;

/* max size for content/virus scanning */
unsigned int maxsize;
unsigned int pfilter_max;

/* pointer to the infos from squid @ fd=0 */
char *squid_url;
char *squid_ip;
char *squid_host;
char *squid_user;
char *squid_method;

/* lists */
LIST_HEAD(hosts);
LIST_HEAD(users);
LIST_HEAD(bfilter);
LIST_HEAD(bcfilter);
LIST_HEAD(brfilter);
LIST_HEAD(cfilter);
LIST_HEAD(crfilter);
LIST_HEAD(pfilter);
LIST_HEAD(ufilter);
LIST_HEAD(urfilter);
LIST_HEAD(wfilter);
LIST_HEAD(wrfilter);

/* defined filters of current ip / host */
stralloc *bfilter_host;
stralloc *bcfilter_host;
stralloc *brfilter_host;
stralloc *cfilter_host;
stralloc *crfilter_host;
stralloc *pfilter_host;
stralloc *ufilter_host;
stralloc *urfilter_host;
stralloc *wfilter_host;
stralloc *wrfilter_host;

/* defined filters of current user */
stralloc *bfilter_user;
stralloc *bcfilter_user;
stralloc *brfilter_user;
stralloc *cfilter_user;
stralloc *crfilter_user;
stralloc *pfilter_user;
stralloc *ufilter_user;
stralloc *urfilter_user;
stralloc *wfilter_user;
stralloc *wrfilter_user;

/* destination websites for squid */
char *redir_badurl;
char *redir_badcont;
char *redir_baduser;
char *redir_badhost;
char *redir_safesite;
char *redir_virusfound;
char *redir_bannerfound;
char *redir_webinterface;

/* misc directories */
char *host_dir;
char *user_dir;
char *bfilter_dir;
char *bcfilter_dir;
char *brfilter_dir;
char *cfilter_dir;
char *crfilter_dir;
char *pfilter_dir;
char *ufilter_dir;
char *urfilter_dir;
char *wfilter_dir;
char *wrfilter_dir;

/* decoded infos */
stralloc ip;
stralloc url;
stralloc host;
stralloc user;
stralloc method;

/* the final destination site */
stralloc redirsite;

/* read configuration environment variables */
void setup(void)
{
	int r;
	char *x;

	/* init the stralloc's */
	stralloc_init(&ip);
	stralloc_init(&url);
	stralloc_init(&host);
	stralloc_init(&user);
	stralloc_init(&method);
	stralloc_init(&redirsite);

	/* open logfile */
	x=env_get("LOG_FILE");
	if (!x) x=LOG_FILE;
	if (!io_appendfile(&fd_log, x)) panic("could not open logfile");

	/* virus warning url */
	redir_virusfound=env_get("REDIR_VIRUSFOUND");
	if (!redir_virusfound) redir_virusfound=REDIR_VIRUSFOUND;

	/* url to a banner image (usally an 1x1 transparent png) */
	redir_bannerfound=env_get("REDIR_BANNERFOUND");
	if (!redir_bannerfound) redir_bannerfound=REDIR_BANNERFOUND;

	/* user is not permitted to use the proxy */
	redir_baduser=env_get("REDIR_BADUSER");
	if (!redir_baduser) redir_baduser=REDIR_BADUSER;

	/* host is not permitted to use the proxy */
	redir_badhost=env_get("REDIR_BADHOST");
	if (!redir_badhost) redir_badhost=REDIR_BADHOST;

	/* url is not okay */
	redir_badurl=env_get("REDIR_BADURL");
	if (!redir_badurl) redir_badurl=REDIR_BADURL;

	/* content is not okay */
	redir_badcont=env_get("REDIR_BADCONT");
	if (!redir_badcont) redir_badhost=REDIR_BADCONT;

	/* this url is can't be denied */
	redir_safesite=env_get("REDIR_SAFESITE");
	if (!redir_safesite) redir_safesite=REDIR_SAFESITE;

	/* this url is used for the configuration of squidwall */
	redir_webinterface=env_get("REDIR_WEBINTERFACE");
	if (!redir_webinterface) redir_webinterface=REDIR_WEBINTERFACE;

	/* directory for the userfiles */
	user_dir=env_get("USERS_DIR");
	if (!user_dir) user_dir=USERS_DIR;

	/* directory for the hostfiles */
	host_dir=env_get("HOSTS_DIR");
	if (!host_dir) host_dir=HOSTS_DIR;

	/* directory with banner/url filters (string) */
	bfilter_dir=env_get("BFILTERS_DIR");
	if (!bfilter_dir) bfilter_dir=BFILTERS_DIR;

	/* directory with banner/url filters (regex) */
	brfilter_dir=env_get("BRFILTERS_DIR");
	if (!brfilter_dir) brfilter_dir=BRFILTERS_DIR;

	/* directory with banner/content filters (regex) */
	bcfilter_dir=env_get("BCFILTERS_DIR");
	if (!bcfilter_dir) bcfilter_dir=BCFILTERS_DIR;

	/* directory with content filters (string) */
	cfilter_dir=env_get("CFILTERS_DIR");
	if (!cfilter_dir) cfilter_dir=CFILTERS_DIR;

	/* directory with content filters (regex) */
	crfilter_dir=env_get("CRFILTERS_DIR");
	if (!crfilter_dir) crfilter_dir=CRFILTERS_DIR;

	/* directory with phrase filters */
	pfilter_dir=env_get("PFILTERS_DIR");
	if (!pfilter_dir) pfilter_dir=PFILTERS_DIR;

	/* directory with url filters (string) */
	ufilter_dir=env_get("UFILTERS_DIR");
	if (!ufilter_dir) ufilter_dir=UFILTERS_DIR;

	/* directory with url filters (regex) */
	urfilter_dir=env_get("URFILTERS_DIR");
	if (!urfilter_dir) urfilter_dir=URFILTERS_DIR;

	/* directory with url whitelists (string) */
	wfilter_dir=env_get("WFILTERS_DIR");
	if (!wfilter_dir) wfilter_dir=WFILTERS_DIR;

	/* directory with url whitelists (regex) */
	wrfilter_dir=env_get("WRFILTERS_DIR");
	if (!wrfilter_dir) wrfilter_dir=WRFILTERS_DIR;

	/* read clamd configuration */
	x=env_get("CLAMD");
	if (!x) x=CLAMD;
	r=ip4_scan(x, (char*)&clamd_ip);
	if (!r) panic("can't get ip of $CLAMD!");
	r=scan_uint(x+r+1, &clamd_port);
	if (!r) panic("can't get port of $CLAMD!");
	
	/* read proxy configuration */
	x=env_get("PROXY");
	if (!x) x=PROXY;
	r=ip4_scan(x, (char*)&proxy_ip);
	if (!r) panic("can't get ip of $PROXY!");
	r=scan_uint(x+r+1, &proxy_port);
	if (!r) panic("can't get port of $PROXY!");

	/* read max filesize for clamd virus scans */
	x=env_get("MAX_SIZE");
	if (!x) {
		maxsize=MAX_SIZE;
	} else {
		r=scan_uint(x, &maxsize);
		if (!r) panic("can't read environment $MAX_SIZE!");
	}

	/* read the deadline for the hitpoints */
	x=env_get("PFILTER_MAX");
	if (!x) {
		pfilter_max=PFILTER_MAX;
	} else {
		r=scan_uint(x, &pfilter_max);
		if (!r) panic("can't read environment $PFILTER_MAX!");
	}

	/* level of debugging */
#if DEBUG_LEVEL
	x=env_get("DEBUG_LEVEL");
	if (!x) {
		debug_level=DEBUG_LEVEL;
	} else {
		r=scan_int(x, &debug_level);
		if (!r) panic("can't read environment $DEBUG_LEVEL!");
	}
#endif

	return;
}

int main(int argc, char **argv)
{
	unsigned int i;
	stralloc sa;
	int r, flags, todo;

	stralloc_init(&sa);
	setup();
	read_acls();

	if (!io_fd(0)) return 111;
	if (!io_fd(1)) return 111;

	for (;;) {
		int state=0;

		/* free old stuff */
		if (!stralloc_copys(&sa, "")) die_nomem();
		if (!stralloc_copys(&ip, "")) die_nomem();
		if (!stralloc_copys(&url, "")) die_nomem();
		if (!stralloc_copys(&host, "")) die_nomem();
		if (!stralloc_copys(&user, "")) die_nomem();
		if (!stralloc_copys(&method, "")) die_nomem();
		if (!stralloc_copys(&redirsite, "")) die_nomem();

		for (;;) {
			io_wantread(0); io_wait(); io_dontwantread(0);
			r=buffer_getline_sa(buffer_0, &sa);
			if (r == 0) return 0;
			if (r == 1) break;
		}
		sa.s[sa.len-1]=0; /* remove the newline */

		/* format: url ip/host user method
		 * sample1: http://www.mcmilk.de/ 192.168.100.10/aspire mcmilk GET
		 * sample2: www.mcmilk.de:443 192.168.100.10/aspire mcmilk GET
		 * state=0 url=http://www.mcmilk.de/
		 * state=1 ip=192.168.100.10
		 * state=2 host=aspire
		 * state=3 user=root
		 * state=4 method=GET
		 */

		squid_url=sa.s;
		for (i=0; i<=sa.len; i++) {
			switch (state) {
			case 0:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=1; squid_ip=sa.s+i+1;
			}
			break;
			case 1:
			if (sa.s[i] == '/') {
				sa.s[i]=0; state=2; squid_host=sa.s+i+1;
			}
			break;
			case 2:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=3; squid_user=sa.s+i+1;
			}
			break;
			case 3:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=4; squid_method=sa.s+i+1;
			}
			break;
			} /* switch() */
		}

		if (state != 4) {
			log_headline("error");
			log_s("incorrect input from squid!\n");
			log_flush();
			goto out;
		}
		
		/* squid gives us urlencoded things - we decode them first! */
		if (!scan_to_sa(scan_urlencoded,  squid_ip, &ip)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_url, &url)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_host, &host)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_user, &user)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_method, &method)) die_nomem();

		ip.s[ip.len]=0;
		url.s[url.len]=0;
		host.s[host.len]=0;
		user.s[user.len]=0;
		method.s[method.len]=0;

		/* we check in this order:
		 *
		 * 1. is the user okay ?
		 * 2. is the ip okay ?
		 * 3. is the host okay ?
		 * 4. look for ad's / banners
		 * 5. look for whitelisted url's
		 * 6. look for bad url's
		 * 7. check for bad content
		 *    - cfilter, crfilter
		 *    - antivirus
		 *    - bcfilter (banner via content filter)
		 */

		/* redirect to the real webinterface */
		if (str_start(squid_url, "http://squidwall")) {
			redir_site(REDIR_TYPE_CONFIG);
			goto out;
		}

		/* just pass through ... */
		if (str_start(squid_url, redir_safesite)) {
			goto out;
		}

		flags=0; todo=0;

		/* read/recheck filter definitions and acl's */
		read_acls();
		
		/* is the user okay ? */
		flags |= check_user(&todo);
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_USER);
			goto out;
		}

		/* is the host/ip okay ? */
		flags |= check_host(&todo);
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_HOST);
			goto out;
		}

		/* check for banner */
		flags |= check_banner();
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_BANNER);
			goto out;
		}

		/* check for whitelisted url's */
		if (check_wlist() == ACL_ALLOW) {
			goto out;
		}

		/* check for blacklisted url's */
		flags |= check_url();
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_URL);
			goto out;
		}

		/* last step: scan the content */
		if (isset(flags, ACL_FILTER)) {
			check_cont(todo);
		}

		out:

		/* give squid an empty newline '\n' or the whole redirsite */
		io_wantwrite(1); io_wait(); io_dontwantwrite(1);
		buffer_putsa(buffer_1, &redirsite);
		buffer_puts(buffer_1, "\n");
		log_flush();
		while (buffer_flush(buffer_1) == -1);
	}

	return 0;
}

==> squidwall-0.4a.tar.bz2: squidwall-0.4a/squidwall.c <==
#include "squidwall.h"

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

void setup(void);

/* fd of logfile */
int64 fd_log;

/* ip's and ports */
struct ip_address clamd_ip;
struct ip_address proxy_ip;
unsigned clamd_port;
unsigned clams_port;
unsigned proxy_port;

/* max size for content/virus scanning */
unsigned int maxsize;
unsigned int pfilter_max;

/* pointer to the infos from squid @ fd=0 */
char *squid_url;
char *squid_ip;
char *squid_host;
char *squid_user;
char *squid_method;

/* lists */
LIST_HEAD(hosts);
LIST_HEAD(users);
LIST_HEAD(redirects);
LIST_HEAD(bfilter);
LIST_HEAD(bcfilter);
LIST_HEAD(brfilter);
LIST_HEAD(cfilter);
LIST_HEAD(crfilter);
LIST_HEAD(pfilter);
LIST_HEAD(ufilter);
LIST_HEAD(ufilter_md5);
LIST_HEAD(ufilter_sha1);
LIST_HEAD(urfilter);
LIST_HEAD(wfilter);
LIST_HEAD(wrfilter);

/* defined filters of current ip / host */
stralloc *bfilter_host;
stralloc *bcfilter_host;
stralloc *brfilter_host;
stralloc *cfilter_host;
stralloc *crfilter_host;
stralloc *pfilter_host;
stralloc *ufilter_host;
stralloc *ufilter_md5_host;
stralloc *ufilter_sha1_host;
stralloc *urfilter_host;
stralloc *wfilter_host;
stralloc *wrfilter_host;

/* defined filters of current user */
stralloc *bfilter_user;
stralloc *bcfilter_user;
stralloc *brfilter_user;
stralloc *cfilter_user;
stralloc *crfilter_user;
stralloc *pfilter_user;
stralloc *ufilter_user;
stralloc *ufilter_md5_user;
stralloc *ufilter_sha1_user;
stralloc *urfilter_user;
stralloc *wfilter_user;
stralloc *wrfilter_user;

/* destination websites for squid */
char *redir_badurl;
char *redir_badcont;
char *redir_baduser;
char *redir_badhost;
char *redir_safesite;
char *redir_virusfound;
char *redir_bannerfound;
char *redir_webinterface;

/* misc directories */
char *host_dir;
char *user_dir;
char *redir_dir;
char *bfilter_dir;
char *bcfilter_dir;
char *brfilter_dir;
char *cfilter_dir;
char *crfilter_dir;
char *pfilter_dir;
char *ufilter_dir;
char *ufilter_md5_dir;
char *ufilter_sha1_dir;
char *urfilter_dir;
char *wfilter_dir;
char *wrfilter_dir;

/* clamd socket */
char *clamav_socket;

/* decoded infos */
stralloc ip;
stralloc url;
stralloc host;
stralloc user;
stralloc method;

/* the final destination site */
stralloc redirsite;

/* infors for the redirect site */
stralloc clamav;
stralloc filter;
stralloc virus;

/* read configuration environment variables */
void setup(void)
{
	int r;
	char *x;

	/* init the stralloc's */
	stralloc_init(&ip);
	stralloc_init(&url);
	stralloc_init(&host);
	stralloc_init(&user);
	stralloc_init(&method);
	stralloc_init(&redirsite);

	stralloc_init(&clamav);
	stralloc_init(&filter);
	stralloc_init(&virus);

	/* open logfile */
	x=env_get("LOG_FILE");
	if (!x) x=LOG_FILE;
	if (!io_appendfile(&fd_log, x)) panic("could not open logfile");

	/* virus warning url */
	redir_virusfound=env_get("REDIR_VIRUSFOUND");
	if (!redir_virusfound) redir_virusfound=REDIR_VIRUSFOUND;

	/* url to a banner image (usally an 1x1 transparent png) */
	redir_bannerfound=env_get("REDIR_BANNERFOUND");
	if (!redir_bannerfound) redir_bannerfound=REDIR_BANNERFOUND;

	/* user is not permitted to use the proxy */
	redir_baduser=env_get("REDIR_BADUSER");
	if (!redir_baduser) redir_baduser=REDIR_BADUSER;

	/* host is not permitted to use the proxy */
	redir_badhost=env_get("REDIR_BADHOST");
	if (!redir_badhost) redir_badhost=REDIR_BADHOST;

	/* url is not okay */
	redir_badurl=env_get("REDIR_BADURL");
	if (!redir_badurl) redir_badurl=REDIR_BADURL;

	/* content is not okay */
	redir_badcont=env_get("REDIR_BADCONT");
	if (!redir_badcont) redir_badhost=REDIR_BADCONT;

	/* this url is can't be denied */
	redir_safesite=env_get("REDIR_SAFESITE");
	if (!redir_safesite) redir_safesite=REDIR_SAFESITE;

	/* this url is used for the configuration of squidwall */
	redir_webinterface=env_get("REDIR_WEBINTERFACE");
	if (!redir_webinterface) redir_webinterface=REDIR_WEBINTERFACE;

	/* directory for the userfiles */
	user_dir=env_get("USERS_DIR");
	if (!user_dir) user_dir=USERS_DIR;

	/* directory for the hostfiles */
	host_dir=env_get("HOSTS_DIR");
	if (!host_dir) host_dir=HOSTS_DIR;

	/* directory with the symbolic links for redirecting sites */
	redir_dir=env_get("REDIR_DIR");
	if (!redir_dir) redir_dir=REDIR_DIR;

	/* directory with banner/url filters (string) */
	bfilter_dir=env_get("BFILTERS_DIR");
	if (!bfilter_dir) bfilter_dir=BFILTERS_DIR;

	/* directory with banner/url filters (regex) */
	brfilter_dir=env_get("BRFILTERS_DIR");
	if (!brfilter_dir) brfilter_dir=BRFILTERS_DIR;

	/* directory with banner/content filters (regex) */
	bcfilter_dir=env_get("BCFILTERS_DIR");
	if (!bcfilter_dir) bcfilter_dir=BCFILTERS_DIR;

	/* directory with content filters (string) */
	cfilter_dir=env_get("CFILTERS_DIR");
	if (!cfilter_dir) cfilter_dir=CFILTERS_DIR;

	/* directory with content filters (regex) */
	crfilter_dir=env_get("CRFILTERS_DIR");
	if (!crfilter_dir) crfilter_dir=CRFILTERS_DIR;

	/* directory with phrase filters */
	pfilter_dir=env_get("PFILTERS_DIR");
	if (!pfilter_dir) pfilter_dir=PFILTERS_DIR;

	/* directory with url filters (string) */
	ufilter_dir=env_get("UFILTERS_DIR");
	if (!ufilter_dir) ufilter_dir=UFILTERS_DIR;

	/* directory with url filters (md5 string) */
	ufilter_md5_dir=env_get("UFILTERS_MD5_DIR");
	if (!ufilter_md5_dir) ufilter_md5_dir=UFILTERS_MD5_DIR;

	/* directory with url filters (sha1 string) */
	ufilter_sha1_dir=env_get("UFILTERS_SHA1_DIR");
	if (!ufilter_sha1_dir) ufilter_sha1_dir=UFILTERS_SHA1_DIR;

	/* directory with url filters (regex) */
	urfilter_dir=env_get("URFILTERS_DIR");
	if (!urfilter_dir) urfilter_dir=URFILTERS_DIR;

	/* directory with url whitelists (string) */
	wfilter_dir=env_get("WFILTERS_DIR");
	if (!wfilter_dir) wfilter_dir=WFILTERS_DIR;

	/* directory with url whitelists (regex) */
	wrfilter_dir=env_get("WRFILTERS_DIR");
	if (!wrfilter_dir) wrfilter_dir=WRFILTERS_DIR;

	/* read clamd tcp:port, or take the default */
	x=env_get("CLAMD");
	if (!x) x=CLAMD;
	r=ip4_scan(x, (char*)&clamd_ip);
	if (!r) panic("can't get ip of $CLAMD!");
	r=scan_uint(x+r+1, &clamd_port);
	if (!r) panic("can't get port of $CLAMD!");

	/* read clamd socket file, or take the default */
	clamav_socket=env_get("CLAMAV_SOCKET");
	if (!clamav_socket) clamav_socket=CLAMAV_SOCKET;
	
	/* read proxy configuration */
	x=env_get("PROXY");
	if (!x) x=PROXY;
	r=ip4_scan(x, (char*)&proxy_ip);
	if (!r) panic("can't get ip of $PROXY!");
	r=scan_uint(x+r+1, &proxy_port);
	if (!r) panic("can't get port of $PROXY!");

	/* read max filesize for clamd virus scans */
	x=env_get("MAX_SIZE");
	if (!x) {
		maxsize=MAX_SIZE;
	} else {
		r=scan_uint(x, &maxsize);
		if (!r) panic("can't read environment $MAX_SIZE!");
	}

	/* read the deadline for the hitpoints */
	x=env_get("PFILTER_MAX");
	if (!x) {
		pfilter_max=PFILTER_MAX;
	} else {
		r=scan_uint(x, &pfilter_max);
		if (!r) panic("can't read environment $PFILTER_MAX!");
	}

	/* level of debugging */
#if DEBUG_LEVEL
	x=env_get("DEBUG_LEVEL");
	if (!x) {
		debug_level=DEBUG_LEVEL;
	} else {
		r=scan_int(x, &debug_level);
		if (!r) panic("can't read environment $DEBUG_LEVEL!");
	}
#endif

	return;
}

/* int main(int argc, char **argv) */
int main(void)
{
	unsigned int i;
	stralloc sa;
	int r, flags, todo;

	stralloc_init(&sa);
	setup();
	read_acls();
	log_flush();

	if (!io_fd(0)) return 111;
	if (!io_fd(1)) return 111;

	for (;;) {
		int state=0;

		/* free old stuff */
		if (!stralloc_copys(&sa, "")) die_nomem();
		if (!stralloc_copys(&ip, "")) die_nomem();
		if (!stralloc_copys(&url, "")) die_nomem();
		if (!stralloc_copys(&host, "")) die_nomem();
		if (!stralloc_copys(&user, "")) die_nomem();
		if (!stralloc_copys(&method, "")) die_nomem();
		if (!stralloc_copys(&redirsite, "")) die_nomem();

		for (;;) {
			io_wantread(0); io_wait(); io_dontwantread(0);
			r=buffer_getline_sa(buffer_0, &sa);
			if (r == 0) return 0;
			if (r == 1) break;
		}
		sa.s[sa.len-1]=0; /* remove the newline */

		/* format: url ip/host user method
		 * sample1: http://www.mcmilk.de/ 192.168.100.10/aspire mcmilk GET
		 * sample2: www.mcmilk.de:443 192.168.100.10/aspire mcmilk GET
		 * state=0 url=http://www.mcmilk.de/
		 * state=1 ip=192.168.100.10
		 * state=2 host=aspire
		 * state=3 user=root
		 * state=4 method=GET
		 */

		squid_url=sa.s;
		for (i=0; i<=sa.len; i++) {
			switch (state) {
			case 0:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=1; squid_ip=sa.s+i+1;
			}
			break;
			case 1:
			if (sa.s[i] == '/') {
				sa.s[i]=0; state=2; squid_host=sa.s+i+1;
			}
			break;
			case 2:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=3; squid_user=sa.s+i+1;
			}
			break;
			case 3:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=4; squid_method=sa.s+i+1;
			}
			break;
			} /* switch() */
		}

		if (state != 4) {
			log_headline("error");
			log_s("incorrect input from squid!\n");
			log_flush();
			goto out;
		}
		
		/* squid gives us urlencoded things - we decode them first! */
		if (!scan_to_sa(scan_urlencoded,  squid_ip, &ip)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_url, &url)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_host, &host)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_user, &user)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_method, &method)) die_nomem();

		ip.s[ip.len]=0;
		url.s[url.len]=0;
		host.s[host.len]=0;
		user.s[user.len]=0;
		method.s[method.len]=0;

		/* we check in this order:
		 *
		 *
		 * 1  is the url http://squidwall -> redirect to webinterface
		 * 2  check for redirect sites -> redirect if needed
		 * 3. is the user okay ?
		 * 4. is the ip okay ?
		 * 5. is the host okay ?
		 * 6. look for ad's / banners
		 * 7. look for whitelisted url's
		 * 8. look for blacklisted url's
		 * 9. check for bad content
		 *    - pfilter, cfilter, crfilter
		 *    - antivirus
		 *    - bcfilter (banner via content regex filter)
		 */

		/* redirect to the real webinterface */
		if (str_start(squid_url, "http://squidwall")) {
			redir_site(REDIR_TYPE_CONFIG);
			goto out;
		}

		/* just pass through */
		if (str_start(squid_url, redir_safesite)) {
			goto out;
		}

		flags=0; todo=0;

		/* recheck the filter definitions and acl's */
		read_acls();

		/* check, if the url matches some user defined redirect site */
		if (check_redirect()) {
			/* stralloc redirsite is set in the function */
			goto out;
		}

		/* is the user okay ? */
		flags |= check_user(&todo);
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_USER);
			goto out;
		}

		/* is the host/ip okay ? */
		flags |= check_host(&todo);
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_HOST);
			goto out;
		}

		/* check for banner */
		flags |= check_banner();
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_BANNER);
			goto out;
		}

		/* check for whitelisted url's */
		if (check_wlist() == ACL_ALLOW) {
			goto out;
		}

		/* check for blacklisted url's */
		flags |= check_blist();
		if (isset(flags, ACL_DENY)) {
			redir_site(REDIR_TYPE_URL);
			goto out;
		}

		/* last step: scan the content */
		if (isset(flags, ACL_FILTER)) {
			check_cont(todo);
		}

		out:

		/* give squid an empty newline '\n' or the whole redirsite */
		io_wantwrite(1); io_wait(); io_dontwantwrite(1);
		buffer_putsa(buffer_1, &redirsite);
		buffer_puts(buffer_1, "\n");
		log_flush();
		while (buffer_flush(buffer_1) == -1);
	}

	return 0;
}
